# Generated by Django 2.2 on 2019-04-03 10:26

import jsonfield.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Plugin",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50)),
                ("plugin_name", models.CharField(max_length=50)),
                ("plugin_type", models.CharField(max_length=50)),
                ("enabled", models.BooleanField(default=False)),
                ("last_update", models.DateTimeField(auto_now=True)),
                ("config", jsonfield.fields.JSONField(blank=True, default={})),
            ],
            options={"ordering": ("pk",), "unique_together": {("name", "plugin_type")}},
        )
    ]
