# Generated by Django 2.0.4 on 2019-07-15 19:44

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("unplugged", "0007_auto_20190706_0817")]

    operations = [
        migrations.RemoveField(
            model_name="simpleadminplugin", name="auto_created_plugins"
        ),
        migrations.RemoveField(model_name="simpleadminplugin", name="plugin"),
        migrations.RemoveField(model_name="simpleadminplugin", name="template"),
        migrations.DeleteModel(name="SimpleAdminTemplate"),
        migrations.AlterModelOptions(
            name="logmessage", options={"ordering": ("-datetime",)}
        ),
        migrations.AlterField(
            model_name="logmessage",
            name="log",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="log_messages",
                to="unplugged.Log",
            ),
        ),
        migrations.DeleteModel(name="SimpleAdminPlugin"),
    ]
